@@ -421,11 +421,11 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
         goto fail;
     }
     /* Create the DCT encoder state. */
-    jcdp->template = s_DCTE_template;
-    s_init_state((stream_state *)&state, &jcdp->template, 0);
-    if (state.template->set_defaults) {
+    jcdp->templat = s_DCTE_template;
+    s_init_state((stream_state *)&state, &jcdp->templat, 0);
+    if (state.templat->set_defaults) {
         state.memory = mem;
-        (*state.template->set_defaults) ((stream_state *) & state);
+        (*state.templat->set_defaults) ((stream_state *) & state);
         state.memory = NULL;
     }
     state.QFactor = 1.0;	/* disable quality adjustment in zfdcte.c */
@@ -436,6 +436,14 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
     state.Markers.data = 0;
     state.Markers.size = 0;
     state.data.compress = jcdp;
+    /* Add in ICC profile */
+    state.icc_profile = NULL; /* In case it is not set here */
+    if (pdev->icc_struct != NULL && pdev->icc_struct->device_profile[0] != NULL) {
+        cmm_profile_t *icc_profile = pdev->icc_struct->device_profile[0];
+        if (icc_profile->num_comps == pdev->color_info.num_components) {
+            state.icc_profile = icc_profile;
+        }
+    } 
     jcdp->memory = state.jpeg_memory = mem;
     if ((code = gs_jpeg_create_compress(&state)) < 0)
         goto fail;
@@ -478,15 +486,15 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
     /* Make sure we get at least a full scan line of input. */
     state.scan_line_size = jcdp->cinfo.input_components *
         jcdp->cinfo.image_width;
-    jcdp->template.min_in_size =
+    jcdp->templat.min_in_size =
         max(s_DCTE_template.min_in_size, state.scan_line_size);
     /* Make sure we can write the user markers in a single go. */
-    jcdp->template.min_out_size =
+    jcdp->templat.min_out_size =
         max(s_DCTE_template.min_out_size, state.Markers.size);
 
     /* Set up the streams. */
-    fbuf_size = max(512 /* arbitrary */ , jcdp->template.min_out_size);
-    jbuf_size = jcdp->template.min_in_size;
+    fbuf_size = max(512 /* arbitrary */ , jcdp->templat.min_out_size);
+    jbuf_size = jcdp->templat.min_in_size;
     if ((fbuf = gs_alloc_bytes(mem, fbuf_size, "jpeg_print_page(fbuf)")) == 0 ||
         (jbuf = gs_alloc_bytes(mem, jbuf_size, "jpeg_print_page(jbuf)")) == 0
         ) {
@@ -499,10 +507,10 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
     s_std_init(&jstrm, jbuf, jbuf_size, &s_filter_write_procs,
                s_mode_write);
     jstrm.state = (stream_state *) & state;
-    jstrm.procs.process = state.template->process;
+    jstrm.procs.process = state.templat->process;
     jstrm.strm = &fstrm;
-    if (state.template->init)
-        (*state.template->init) (jstrm.state);
+    if (state.templat->init)
+        (*state.templat->init) (jstrm.state);
 
     /* Copy the data to the output. */
     for (lnum = 0; lnum < pdev->height; ++lnum) {
@@ -534,3 +542,4 @@ jpeg_print_page(gx_device_printer * pdev, FILE * prn_stream)
     gs_free_object(mem, in, "jpeg_print_page(in)");
     return code;
 }
+
