@@ -281,9 +281,9 @@ gdev_prn_allocate(gx_device *pdev, gdev_prn_space_params *new_space_params,
             (&buf_space, pdev, NULL, pdev->height, false) >= 0;
         mem_space = buf_space.bits + buf_space.line_ptrs;
         if (ppdev->page_uses_transparency) {
-            pdf14_trans_buffer_size = (ESTIMATED_PDF14_ROW_SPACE(max(1, new_width)) >> 3);
+            pdf14_trans_buffer_size = (ESTIMATED_PDF14_ROW_SPACE(max(1, pdev->width), pdev->color_info.num_components) >> 3);
             if (new_height < (max_ulong - mem_space) / pdf14_trans_buffer_size) {
-                pdf14_trans_buffer_size *= new_height;
+                pdf14_trans_buffer_size *= pdev->height;
                 mem_space += pdf14_trans_buffer_size;
             } else {
                 size_ok = 0;
@@ -720,6 +720,8 @@ label:\
         if (ppdev->file != NULL)
             gx_device_close_output_file(pdev, ppdev->fname, ppdev->file);
         ppdev->file = NULL;
+        if (sizeof(ppdev->fname) <= ofs.size)
+            return_error(gs_error_limitcheck);
         memcpy(ppdev->fname, ofs.data, ofs.size);
         ppdev->fname[ofs.size] = 0;
     }
@@ -1277,6 +1279,31 @@ gdev_prn_close_printer(gx_device * pdev)
     return 0;
 }
 
+/* compare two space_params, we can't do this with memcmp since there is padding in the structure */
+static int
+compare_gdev_prn_space_params(const gdev_prn_space_params sp1, 
+                              const gdev_prn_space_params sp2) {
+  if (sp1.MaxBitmap != sp2.MaxBitmap)
+    return(1);
+  if (sp1.BufferSpace != sp2.BufferSpace)
+    return(1);
+  if (sp1.band.page_uses_transparency != sp2.band.page_uses_transparency)
+    return(1);
+  if (sp1.band.BandWidth != sp2.band.BandWidth)
+    return(1);
+  if (sp1.band.BandHeight != sp2.band.BandHeight)
+    return(1);
+  if (sp1.band.BandBufferSpace != sp2.band.BandBufferSpace)
+    return(1);
+  if (sp1.params_are_read_only != sp2.params_are_read_only)
+    return(1);
+  if (sp1.banding_type != sp2.banding_type)
+    return(1);
+ 
+  return(0); 
+}
+
+
 /* If necessary, free and reallocate the printer memory after changing params */
 int
 gdev_prn_maybe_realloc_memory(gx_device_printer *prdev,
@@ -1297,7 +1324,7 @@ gdev_prn_maybe_realloc_memory(gx_device_printer *prdev,
      * for these filesets.
      */
     if (prdev->is_open &&
-        (memcmp(&prdev->space_params, old_sp, sizeof(*old_sp)) != 0 ||
+        (compare_gdev_prn_space_params(prdev->space_params, *old_sp) != 0 ||
          prdev->width != old_width || prdev->height != old_height ||
          prdev->page_uses_transparency != old_page_uses_transparency)
         ) {
