@@ -403,14 +403,15 @@ static gx_color_index
 rinkj_encode_color(gx_device *dev, const gx_color_value colors[])
 {
     int bpc = ((rinkj_device *)dev)->bitspercomponent;
-    int drop = sizeof(gx_color_value) * 8 - bpc;
     gx_color_index color = 0;
     int i = 0;
     int ncomp = dev->color_info.num_components;
+    COLROUND_VARS;
 
+    COLROUND_SETUP(bpc);
     for (; i<ncomp; i++) {
         color <<= bpc;
-        color |= (colors[i] >> drop);
+        color |= COLROUND_ROUND(colors[i]);
     }
     return (color == gx_no_color_index ? color ^ 1 : color);
 }
@@ -1086,9 +1087,8 @@ rinkj_write_image_data(gx_device_printer *pdev, RinkjDevice *cmyk_dev)
                 if (cache[hash].key != color) {
 
                     /* 3 channel to CMYK */
-                    gscms_transform_color(rdev->icc_link, &cbuf,
-                        &(vbuf), 1, NULL);
-
+                    gscms_transform_color((gx_device *)rdev, rdev->icc_link, 
+                                          &cbuf, &(vbuf), 1);
                     cache[hash].key = color;
                     cache[hash].value = ((bits32 *)vbuf)[0];
 
@@ -1113,9 +1113,8 @@ rinkj_write_image_data(gx_device_printer *pdev, RinkjDevice *cmyk_dev)
                     ((bits32 *)cbuf)[0] = color;
 
                     /* 4 channel to CMYK */
-                    gscms_transform_color(rdev->icc_link, &cbuf,
-                        &(vbuf), 1, NULL);
-
+                    gscms_transform_color((gx_device *)rdev, rdev->icc_link, 
+                                           &cbuf, &(vbuf), 1);
                     cache[hash].key = color;
                     cache[hash].value = ((bits32 *)vbuf)[0];
                 } else {
@@ -1146,9 +1145,8 @@ rinkj_write_image_data(gx_device_printer *pdev, RinkjDevice *cmyk_dev)
                        code was still working with 4 to 4
                        conversion.  Replacing with new ICC AMP call */
 
-                    gscms_transform_color(rdev->icc_link, &cbuf,
-                        &(vbuf), 1, NULL);
-
+                    gscms_transform_color((gx_device *) rdev, rdev->icc_link, 
+                                          &cbuf, &(vbuf), 1);
                     cache[hash].key = color;
                     cache[hash].value = ((bits32 *)vbuf)[0];
                 } else {
