@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gp_os2.h,v 1.2 2010/07/10 22:02:26 Arabidopsis Exp $ */
+/* $Id$ */
 /* Common platform-specific routines for OS/2 */
 
 #ifndef gp_os2_INCLUDED
@@ -21,12 +21,11 @@
 /* If strlen(queue_name)==0, return default queue and driver name */
 /* If queue_name supplied, return driver_name */
 /* returns 0 if OK, non-zero for error */
-int pm_find_queue(char *queue_name, char *driver_name);
+int pm_find_queue(const gs_memory_t *mem, char *queue_name, char *driver_name);
 
 /* Spool file to queue */
 /* Return 0 if successful, non-zero if error. */
 /* If filename is NULL, return 0 if spool queue is valid, non-zero if error */
-int pm_spool(char *filename, const char *queue);
-
+int pm_spool(const gs_memory_t *mem, char *filename, const char *queue);
 
 #endif /* gp_os2_INCLUDED */
