@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxclzlib.c,v 1.2 2010/07/10 22:02:17 Arabidopsis Exp $ */
+/* $Id$ */
 /* zlib filter initialization for RAM-based band lists */
 /* Must be compiled with -I$(ZSRCDIR) */
 #include "std.h"
@@ -20,30 +20,28 @@
 #include "gxclmem.h"
 #include "szlibx.h"
 
-static stream_zlib_state cl_zlibE_state;
-static stream_zlib_state cl_zlibD_state;
-
-/* Initialize the states to be copied. */
-int
-gs_cl_zlib_init(gs_memory_t * mem)
-{
-    s_zlib_set_defaults((stream_state *) & cl_zlibE_state);
-    cl_zlibE_state.no_wrapper = true;
-    cl_zlibE_state.template = &s_zlibE_template;
-    s_zlib_set_defaults((stream_state *) & cl_zlibD_state);
-    cl_zlibD_state.no_wrapper = true;
-    cl_zlibD_state.template = &s_zlibD_template;
-    return 0;
-}
-
 /* Return the prototypes for compressing/decompressing the band list. */
-const stream_state *
-clist_compressor_state(void *client_data)
+const stream_template *
+clist_compressor_template(void)
 {
-    return (const stream_state *)&cl_zlibE_state;
+    return &s_zlibE_template;
 }
-const stream_state *
-clist_decompressor_state(void *client_data)
+const stream_template *
+clist_decompressor_template(void)
 {
-    return (const stream_state *)&cl_zlibD_state;
+    return &s_zlibD_template;
+}
+void
+clist_compressor_init(stream_state *state)
+{
+    s_zlib_set_defaults(state);
+    ((stream_zlib_state *)state)->no_wrapper = true;
+    state->templat = &s_zlibE_template;
+}
+void
+clist_decompressor_init(stream_state *state)
+{
+    s_zlib_set_defaults(state);
+    ((stream_zlib_state *)state)->no_wrapper = true;
+    state->templat = &s_zlibD_template;
 }
