@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxdcconv.h,v 1.2 2010/07/10 22:02:18 Arabidopsis Exp $ */
+/* $Id$ */
 /* Internal device color conversion interfaces */
 
 #ifndef gxdcconv_INCLUDED
@@ -21,12 +21,14 @@
 
 /* Color space conversion routines */
 frac color_rgb_to_gray(frac r, frac g, frac b,
-		       const gs_imager_state * pis);
+                       const gs_imager_state * pis);
 void color_rgb_to_cmyk(frac r, frac g, frac b,
-		       const gs_imager_state * pis, frac cmyk[4]);
+                       const gs_imager_state * pis, frac cmyk[4],
+                       gs_memory_t * mem);
 frac color_cmyk_to_gray(frac c, frac m, frac y, frac k,
-			const gs_imager_state * pis);
+                        const gs_imager_state * pis);
 void color_cmyk_to_rgb(frac c, frac m, frac y, frac k,
-		       const gs_imager_state * pis, frac rgb[3]);
+                       const gs_imager_state * pis, frac rgb[3],
+                       gs_memory_t * mem);
 
 #endif /* gxdcconv_INCLUDED */
