@@ -64,7 +64,7 @@
 /*
  * I/O Stream Class
  *
- * $Id: jas_stream.h,v 1.6 2010/07/10 22:02:56 Arabidopsis Exp $
+ * $Id$
  */
 
 #ifndef JAS_STREAM_H
@@ -76,6 +76,7 @@
 
 #include <jasper/jas_config.h>
 
+#include <limits.h>
 #include <stdio.h>
 #if defined(HAVE_FCNTL_H)
 #include <fcntl.h>
@@ -259,7 +260,11 @@ typedef struct {
 typedef struct {
 	int fd;
 	int flags;
-	char pathname[L_tmpnam + 1];
+#ifndef _WIN32
+	char pathname[PATH_MAX + 1];
+#else
+	char pathname[_MAX_PATH + 1];
+#endif
 } jas_stream_fileobj_t;
 
 #define	JAS_STREAM_FILEOBJ_DELONCLOSE	0x01
