@@ -180,8 +180,13 @@ jbig2_arith_new (Jbig2Ctx *ctx, Jbig2WordStream *ws)
 {
   Jbig2ArithState *result;
 
-  result = (Jbig2ArithState *)jbig2_alloc(ctx->allocator,
-	sizeof(Jbig2ArithState));
+  result = jbig2_new(ctx, Jbig2ArithState, 1);
+  if (result == NULL)
+  {
+      jbig2_error(ctx, JBIG2_SEVERITY_FATAL, -1,
+          "failed to allocate Jbig2ArithState in jbig2_arith_new");
+      return result;
+  }
 
   result->ws = ws;
 
