@@ -16,7 +16,13 @@
 /* library internals */
 
 typedef uint8_t byte;
-typedef int bool;
+
+#define bool int
+
+#ifdef __cplusplus
+#define template template_C
+#define new new_C
+#endif
 
 #ifndef TRUE
 #define TRUE 1
@@ -67,9 +73,15 @@ struct _Jbig2Ctx {
   Jbig2Page *pages;
 };
 
+uint32_t
+jbig2_get_uint32(const byte *bptr);
+
 int32_t
 jbig2_get_int32 (const byte *buf);
 
+uint16_t
+jbig2_get_uint16(const byte *bptr);
+
 int16_t
 jbig2_get_int16 (const byte *buf);
 
