@@ -1,23 +1,17 @@
 /* Copyright (C) 2001-2006 Artifex Software, Inc.
    All Rights Reserved.
-  
-  This file is part of GNU ghostscript
 
-  GNU ghostscript is free software; you can redistribute it and/or
-  modify it under the terms of the version 2 of the GNU General Public
-  License as published by the Free Software Foundation.
-
-  GNU ghostscript is distributed in the hope that it will be useful, but WITHOUT
-  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-  You should have received a copy of the GNU General Public License along with
-  ghostscript; see the file COPYING. If not, write to the Free Software Foundation,
-  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+   This software is provided AS-IS with no warranty, either express or
+   implied.
 
+   This software is distributed under license and may not be copied, modified
+   or distributed except as expressly authorized under the terms of that
+   license.  Refer to licensing information at http://www.artifex.com/
+   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
+   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: zcolor2.c,v 1.8 2008/03/23 15:27:41 Arabidopsis Exp $ */
+/* $Id$ */
 /* Level 2 color operators */
 #include "ghost.h"
 #include "string_.h"
@@ -29,7 +23,6 @@
 #include "igstate.h"
 #include "store.h"
 
-
 /*
  *  -   .useralternate   <bool>
  *
@@ -48,7 +41,6 @@ zusealternate(i_ctx_t * i_ctx_p)
     return 0;
 }
 
-
 /* ------ Initialization procedure ------ */
 
 const op_def    zcolor2_l2_op_defs[] = {
