@@ -53,7 +53,7 @@ zCFE(i_ctx_t *i_ctx_p)
 
 int
 filter_write_predictor(i_ctx_t *i_ctx_p, int npop,
-                       const stream_template * template, stream_state * st)
+                       const stream_template * templat, stream_state * st)
 {
     os_ptr op = osp;
     int predictor, code;
@@ -88,7 +88,7 @@ filter_write_predictor(i_ctx_t *i_ctx_p, int npop,
     } else
         predictor = 1;
     if (predictor == 1)
-        return filter_write(i_ctx_p, npop, template, st, 0);
+        return filter_write(i_ctx_p, npop, templat, st, 0);
     {
         /* We need to cascade filters. */
         ref rtarget, rdict;
@@ -97,7 +97,7 @@ filter_write_predictor(i_ctx_t *i_ctx_p, int npop,
         /* Save the operands, just in case. */
         ref_assign(&rtarget, op - 1);
         ref_assign(&rdict, op);
-        code = filter_write(i_ctx_p, npop, template, st, 0);
+        code = filter_write(i_ctx_p, npop, templat, st, 0);
         if (code < 0)
             return code;
         /* filter_write changed osp.... */
